@@ -13,7 +13,8 @@ struct static_config {
   struct proto_config c;
   list iface_routes;		/* Routes to search on interface events */
   list other_routes;		/* Routes hooked to neighbor cache and reject routes */
-  int check_link;		/* Whether iface link state is used */
+  int check_link;			/* Whether iface link state is used */
+  struct rtable_config *igp_table;	/* Table used for recursive next hop lookups */
 };
 
 
@@ -35,6 +36,9 @@ struct static_route {
 /* Dummy nodes (parts of multipath route) abuses masklen field for weight
    and if_name field for a ptr to the master (RTD_MULTIPATH) node. */
 
+
+#define RTDX_RECURSIVE 0x7f		/* Phony dest value for recursive routes */
+
 void static_show(struct proto *);
 
 #endif
