@@ -19,7 +19,7 @@ struct kif_proto;
 #include "lib/krt-set.h"
 #include "lib/krt-iface.h"
 
-/* Flags stored in net->n.flags */
+/* Flags stored in net->n.flags, rest are in nest/route.h */
 
 #define KRF_VERDICT_MASK 0x0f
 #define KRF_CREATE 0			/* Not seen in kernel table */
@@ -28,9 +28,6 @@ struct kif_proto;
 #define KRF_DELETE 3			/* Should be deleted */
 #define KRF_IGNORE 4			/* To be ignored */
 
-#define KRF_INSTALLED 0x80		/* This route should be installed in the kernel */
-
-
 #define EA_KRT_PREFSRC EA_CODE(EAP_KRT, 0)
 #define EA_KRT_REALM EA_CODE(EAP_KRT, 1)
 
